# coding=utf-8
"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class FaxList(ListResource):

    def __init__(self, version):
        """
        Initialize the FaxList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.fax.v1.fax.FaxList
        :rtype: twilio.rest.fax.v1.fax.FaxList
        """
        super(FaxList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Faxes'.format(**self._solution)

    def stream(self, from_=values.unset, to=values.unset, limit=None,
               page_size=None):
        """
        Streams FaxInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode from_: The from
        :param unicode to: The to
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.fax.v1.fax.FaxInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            from_=from_,
            to=to,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'], limits['page_limit'])

    def list(self, from_=values.unset, to=values.unset, limit=None, page_size=None):
        """
        Lists FaxInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode from_: The from
        :param unicode to: The to
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.fax.v1.fax.FaxInstance]
        """
        return list(self.stream(
            from_=from_,
            to=to,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, from_=values.unset, to=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of FaxInstance records from the API.
        Request is executed immediately

        :param unicode from_: The from
        :param unicode to: The to
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxPage
        """
        params = values.of({
            'From': from_,
            'To': to,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(
            'GET',
            self._uri,
            params=params,
        )

        return FaxPage(self._version, response, self._solution)

    def create(self, from_, to, media_url, quality=values.unset,
               status_callback=values.unset):
        """
        Create a new FaxInstance

        :param unicode from_: The from
        :param unicode to: The to
        :param unicode media_url: The media_url
        :param FaxInstance.Quality quality: The quality
        :param unicode status_callback: The status_callback

        :returns: Newly created FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxInstance
        """
        data = values.of({
            'From': from_,
            'To': to,
            'MediaUrl': media_url,
            'Quality': quality,
            'StatusCallback': status_callback,
        })

        payload = self._version.create(
            'POST',
            self._uri,
            data=data,
        )

        return FaxInstance(
            self._version,
            payload,
        )

    def get(self, sid):
        """
        Constructs a FaxContext

        :param sid: The sid

        :returns: twilio.rest.fax.v1.fax.FaxContext
        :rtype: twilio.rest.fax.v1.fax.FaxContext
        """
        return FaxContext(
            self._version,
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a FaxContext

        :param sid: The sid

        :returns: twilio.rest.fax.v1.fax.FaxContext
        :rtype: twilio.rest.fax.v1.fax.FaxContext
        """
        return FaxContext(
            self._version,
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Fax.V1.FaxList>'


class FaxPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the FaxPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.fax.v1.fax.FaxPage
        :rtype: twilio.rest.fax.v1.fax.FaxPage
        """
        super(FaxPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of FaxInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.fax.v1.fax.FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxInstance
        """
        return FaxInstance(
            self._version,
            payload,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Fax.V1.FaxPage>'


class FaxContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the FaxContext

        :param Version version: Version that contains the resource
        :param sid: The sid

        :returns: twilio.rest.fax.v1.fax.FaxContext
        :rtype: twilio.rest.fax.v1.fax.FaxContext
        """
        super(FaxContext, self).__init__(version)

        # Path Solution
        self._solution = {
            'sid': sid,
        }
        self._uri = '/Faxes/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch a FaxInstance

        :returns: Fetched FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxInstance
        """
        params = values.of({})

        payload = self._version.fetch(
            'GET',
            self._uri,
            params=params,
        )

        return FaxInstance(
            self._version,
            payload,
            sid=self._solution['sid'],
        )

    def update(self, status=values.unset):
        """
        Update the FaxInstance

        :param FaxInstance.UpdateStatus status: The status

        :returns: Updated FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxInstance
        """
        data = values.of({
            'Status': status,
        })

        payload = self._version.update(
            'POST',
            self._uri,
            data=data,
        )

        return FaxInstance(
            self._version,
            payload,
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Fax.V1.FaxContext {}>'.format(context)


class FaxInstance(InstanceResource):

    class Direction(object):
        INBOUND = "inbound"
        OUTBOUND = "outbound"

    class Quality(object):
        STANDARD = "standard"
        FINE = "fine"
        SUPERFINE = "superfine"

    class Status(object):
        QUEUED = "queued"
        PROCESSING = "processing"
        SENDING = "sending"
        DELIVERED = "delivered"
        RECEIVING = "receiving"
        RECEIVED = "received"
        NO_ANSWER = "no-answer"
        BUSY = "busy"
        FAILED = "failed"
        CANCELED = "canceled"

    class UpdateStatus(object):
        CANCELED = "canceled"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the FaxInstance

        :returns: twilio.rest.fax.v1.fax.FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxInstance
        """
        super(FaxInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload['sid'],
            'account_sid': payload['account_sid'],
            'from_': payload['from'],
            'to': payload['to'],
            'quality': payload['quality'],
            'media_url': payload['media_url'],
            'num_pages': deserialize.integer(payload['num_pages']),
            'duration': deserialize.integer(payload['duration']),
            'status': payload['status'],
            'direction': payload['direction'],
            'api_version': payload['api_version'],
            'price': deserialize.decimal(payload['price']),
            'price_unit': payload['price_unit'],
            'date_created': deserialize.iso8601_datetime(payload['date_created']),
            'date_updated': deserialize.iso8601_datetime(payload['date_updated']),
            'url': payload['url'],
        }

        # Context
        self._context = None
        self._solution = {
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: FaxContext for this FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxContext
        """
        if self._context is None:
            self._context = FaxContext(
                self._version,
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The sid
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The account_sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def from_(self):
        """
        :returns: The from
        :rtype: unicode
        """
        return self._properties['from_']

    @property
    def to(self):
        """
        :returns: The to
        :rtype: unicode
        """
        return self._properties['to']

    @property
    def quality(self):
        """
        :returns: The quality
        :rtype: FaxInstance.Quality
        """
        return self._properties['quality']

    @property
    def media_url(self):
        """
        :returns: The media_url
        :rtype: unicode
        """
        return self._properties['media_url']

    @property
    def num_pages(self):
        """
        :returns: The num_pages
        :rtype: unicode
        """
        return self._properties['num_pages']

    @property
    def duration(self):
        """
        :returns: The duration
        :rtype: unicode
        """
        return self._properties['duration']

    @property
    def status(self):
        """
        :returns: The status
        :rtype: FaxInstance.Status
        """
        return self._properties['status']

    @property
    def direction(self):
        """
        :returns: The direction
        :rtype: FaxInstance.Direction
        """
        return self._properties['direction']

    @property
    def api_version(self):
        """
        :returns: The api_version
        :rtype: unicode
        """
        return self._properties['api_version']

    @property
    def price(self):
        """
        :returns: The price
        :rtype: unicode
        """
        return self._properties['price']

    @property
    def price_unit(self):
        """
        :returns: The price_unit
        :rtype: unicode
        """
        return self._properties['price_unit']

    @property
    def date_created(self):
        """
        :returns: The date_created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date_updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch a FaxInstance

        :returns: Fetched FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxInstance
        """
        return self._proxy.fetch()

    def update(self, status=values.unset):
        """
        Update the FaxInstance

        :param FaxInstance.UpdateStatus status: The status

        :returns: Updated FaxInstance
        :rtype: twilio.rest.fax.v1.fax.FaxInstance
        """
        return self._proxy.update(
            status=status,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Fax.V1.FaxInstance {}>'.format(context)
